@@ -48,7 +48,7 @@ bool isDirHidden(QDir& dir)
 
 QString stripTrailingSlash(const QString& path)
 {
-    return path.endsWith('/') ? path.mid(0, path.length()-1) : path;
+    return path.endsWith(QLatin1Char('/')) ? path.mid(0, path.length()-1) : path;
 }
 
 }
@@ -58,8 +58,9 @@ using namespace Baloo;
 FileIndexerConfig::FileIndexerConfig(QObject* parent)
     : QObject(parent)
     , m_config(QStringLiteral("baloofilerc"))
+    , m_folderCacheDirty(true)
     , m_indexHidden(false)
-    , m_devices(new StorageDevices(this))
+    , m_devices(nullptr)
     , m_maxUncomittedFiles(40)
 {
     forceConfigUpdate();
@@ -73,6 +74,8 @@ FileIndexerConfig::~FileIndexerConfig()
 
 QStringList FileIndexerConfig::includeFolders() const
 {
+    const_cast<FileIndexerConfig*>(this)->buildFolderCache();
+
     QStringList fl;
     for (int i = 0; i < m_folderCache.count(); ++i) {
         if (m_folderCache[i].second)
@@ -84,6 +87,8 @@ QStringList FileIndexerConfig::includeFolders() const
 
 QStringList FileIndexerConfig::excludeFolders() const
 {
+    const_cast<FileIndexerConfig*>(this)->buildFolderCache();
+
     QStringList fl;
     for (int i = 0; i < m_folderCache.count(); ++i) {
         if (!m_folderCache[i].second)
@@ -138,6 +143,28 @@ bool FileIndexerConfig::isInitialRun() const
 }
 
 
+bool FileIndexerConfig::canBeSearched(const QString& folder) const
+{
+    QFileInfo fi(folder);
+    QString path = fi.absolutePath();
+    if (!fi.isDir()) {
+        return false;
+    } else if (shouldFolderBeIndexed(path)) {
+        return true;
+    }
+
+    const_cast<FileIndexerConfig*>(this)->buildFolderCache();
+    
+    // Look for included descendants
+    for (const QPair<QString, bool>& fld: m_folderCache) {
+        if (fld.second && fld.first.startsWith(path)) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 bool FileIndexerConfig::shouldBeIndexed(const QString& path) const
 {
     QFileInfo fi(path);
@@ -185,7 +212,7 @@ bool FileIndexerConfig::shouldFolderBeIndexed(const QString& path) const
 
 bool FileIndexerConfig::shouldFileBeIndexed(const QString& fileName) const
 {
-    if (!indexHiddenFilesAndFolders() && fileName.startsWith('.')) {
+    if (!indexHiddenFilesAndFolders() && fileName.startsWith(QLatin1Char('.'))) {
         return false;
     }
     return !m_excludeFilterRegExpCache.exactMatch(fileName);
@@ -205,6 +232,8 @@ bool FileIndexerConfig::folderInFolderList(const QString& path)
 
 bool FileIndexerConfig::folderInFolderList(const QString& path, QString& folder) const
 {
+    const_cast<FileIndexerConfig*>(this)->buildFolderCache();
+
     const QString p = stripTrailingSlash(path);
 
     // we traverse the list backwards to catch all exclude folders
@@ -280,13 +309,22 @@ void cleanupList(QList<QPair<QString, bool> >& result)
 
 void FileIndexerConfig::buildFolderCache()
 {
+    if (!m_folderCacheDirty) {
+        return;
+    }
+
+    if (!m_devices) {
+        m_devices = new StorageDevices(this);
+    }
+
     KConfigGroup group = m_config.group("General");
     QStringList includeFoldersPlain = group.readPathEntry("folders", QStringList() << QDir::homePath());
     QStringList excludeFoldersPlain = group.readPathEntry("exclude folders", QStringList());
 
     // Add all removable media and network shares as ignored unless they have
     // been explicitly added in the include list
-    for (const auto& device: m_devices->allMedia()) {
+    const auto allMedia = m_devices->allMedia();
+    for (const auto& device: allMedia) {
         const QString mountPath = device.mountPath();
         if (!device.isUsable() && !mountPath.isEmpty()) {
             if (!includeFoldersPlain.contains(mountPath)) {
@@ -300,6 +338,8 @@ void FileIndexerConfig::buildFolderCache()
     insertSortFolders(excludeFoldersPlain, false, m_folderCache);
 
     cleanupList(m_folderCache);
+
+    m_folderCacheDirty = false;
 }
 
 
@@ -318,7 +358,7 @@ void FileIndexerConfig::forceConfigUpdate()
 {
     m_config.reparseConfiguration();
 
-    buildFolderCache();
+    m_folderCacheDirty = true;
     buildExcludeFilterRegExpCache();
     buildMimeTypeCache();
 
