@@ -22,6 +22,8 @@
 
 #include "pendingfilequeue.h"
 
+#include <memory>
+
 #include <QDebug>
 #include <QDateTime>
 
@@ -55,6 +57,22 @@ PendingFileQueue::~PendingFileQueue()
 
 void PendingFileQueue::enqueue(const PendingFile& file)
 {
+    // If we get an event to remove /home/A, remove all events for everything under /home/A/
+
+    if (file.shouldRemoveIndex() && file.path().endsWith('/')) {
+        const auto keepFile = [&file](const PendingFile& pending) {
+            return !pending.path().startsWith(file.path());
+        };
+        const auto end = m_cache.end();
+        // std::partition moves all matching entries to the first partition
+        const auto droppedFilesBegin = std::partition(m_cache.begin(), end, keepFile);
+        for (auto it = droppedFilesBegin; it != end; it++) {
+            m_pendingFiles.remove(it->path());
+            m_recentlyEmitted.remove(it->path());
+        }
+        m_cache.erase(droppedFilesBegin, end);
+    }
+
     int i = m_cache.indexOf(file);
     if (i == -1) {
         m_cache << file;
@@ -69,7 +87,7 @@ void PendingFileQueue::processCache()
 {
     QTime currentTime = QTime::currentTime();
 
-    for (const PendingFile& file : m_cache) {
+    for (const PendingFile& file : qAsConst(m_cache)) {
         if (file.shouldRemoveIndex()) {
             Q_EMIT removeFileIndex(file.path());
 
@@ -94,7 +112,11 @@ void PendingFileQueue::processCache()
                 m_pendingFiles[file.path()] = time;
             }
             else {
-                Q_EMIT indexFile(file.path());
+                if (file.isNewFile()) {
+                    Q_EMIT indexNewFile(file.path());
+                } else {
+                    Q_EMIT indexModifiedFile(file.path());
+                }
                 m_recentlyEmitted.insert(file.path(), currentTime);
             }
         } else {
@@ -150,7 +172,7 @@ void PendingFileQueue::processPendingFiles()
 
         int secondsLeft = currentTime.secsTo(it.value());
         if (secondsLeft <= 0) {
-            Q_EMIT indexFile(it.key());
+            Q_EMIT indexModifiedFile(it.key());
             m_recentlyEmitted.insert(it.key(), currentTime);
 
             it.remove();
